/*
Tiny GLUT program to show how textures work.

Orion Sky Lawlor, olawlor@acm.org, 2006/11/09 (Public Domain)
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include <GL/glew.h> /* GL Extensions Wrangler, http://glew.sourceforge.net/ */
#include <GL/glut.h> /* GL Utilities Toolkit, http://freeglut.sourceforge.net/ */

void Exit(const char *where,const char *why) {
	fprintf (stderr, "FATAL OpenGL Error in %s: %s\n", where, why);
	abort();	
}

// Check GL for errors after the last command
void Check(const char *where) {
	GLenum e=glGetError();
	if (e==GL_NO_ERROR) return;
	const GLubyte *errString = gluErrorString(e);
	Exit(where, (const char *)errString);
}

// Draw a little quadrilateral onscreen
void drawBox(void) {
	glBegin(GL_TRIANGLE_FAN);
	glColor4f(1,1,1,1);  
	glTexCoord2f(0,0);   glVertex4f(-1,-1,0,1);  
	glTexCoord2f(0,1);   glVertex4f(-1,+1,0,1);  
	glTexCoord2f(1,1);   glVertex4f(+1,+1,0,1);  
	glTexCoord2f(1,0);   glVertex4f(+1,-1,0,1);
	glEnd();
}

/******** Program Initialization *********/
int win_w, win_h; /**< Width and height (in pixels) of our window at last resize */

/* GLUT calls this routine every time our window needs to be redrawn */
void main_display(void) {
	Check("Entering display routine");

// Clear the color and depth buffers
	glClearColor(0.2,0.3,0.4,1.0); /* set clear color */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); /* actually clear color and depth */
	
// Draw stuff onscreen
	/* Make a program object */
	GLhandleARB prog=glCreateProgramObjectARB();

	/* Make vertex shader object */
	GLhandleARB vert=glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	const char *glsl_code=
	"varying vec4 color;"
	"void main(void) {"
	"	color=gl_Vertex;"
	"	gl_Position=vec4(vec3(0.8),1.0)*gl_Vertex;"
	"}";
	glShaderSourceARB(vert,1,&glsl_code,NULL);
	glCompileShaderARB(vert);
	glAttachObjectARB(prog,vert);

	/* Make fragment shader object */
	GLhandleARB frag=glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
	glsl_code=
	"varying vec4 color;\n"
	"uniform float brightness;\n"
	"void main(void) {\n"
	"	gl_FragColor=color*brightness*vec4(1.0,0.0,0.0,1.0);\n"
	"}";
	glShaderSourceARB(frag,1,&glsl_code ,NULL);
	glCompileShaderARB(frag);
	glAttachObjectARB(prog,frag);

	/* Link the program.  FIXME: check for link errors, with  glGetObjectParameterivARB and glGetInfoLogARB */
	glLinkProgramARB(prog);

	// Here's how to use the program object for rendering:
	glUseProgramObjectARB(prog);
	
	int loc=glGetUniformLocationARB(prog,"brightness");
	glUniform1fARB(loc,-3.0);
	printf("Location of brightenss: %d\n",loc);
	
	drawBox();
	glUseProgramObjectARB(0);  /* reset back to default fixed-function program */

	// Here's how to delete a program object:
	glDeleteObjectARB(vert);
	glDeleteObjectARB(frag);
	glDeleteObjectARB(prog);
	
// Display the stuff drawn so far to the screen:
	glutSwapBuffers();
	Check("Leaving display routine");
}

/* GLUT calls this routine every time our window changes size */
void main_reshape(int w, int h) {
	win_w=w; win_h=h;
	glViewport(0, 0, (GLsizei)w, (GLsizei)(h));
}

int main(int argc, char** argv)
{
	glutInit(&argc,argv);

	// Ask for a back buffer, color buffer, and Z buffer in a window.
	int mode=GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH;
	glutInitDisplayMode (mode);
	int main_window = glutCreateWindow("Bare-metal OpenGL Demo");
	
	// Turn on OpenGL extensions, like programmable shaders
	//   WARNING: you MUST call glutCreateWindow before doing this!
	//   If you don't call glewInit, or call it wrong, you'll segfault!
	glewInit();

	// Ask GLUT to call us when we need to redraw
	glutDisplayFunc (main_display);
	glutReshapeFunc (main_reshape);
	
	// Process events until user closes our window...
	glutMainLoop();
	
	return 0;
}
