/*
Tiny GLUT program to show how textures work.

Orion Sky Lawlor, olawlor@acm.org, 2006/11/09 (Public Domain)
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include <GL/glew.h> /* GL Extensions Wrangler, http://glew.sourceforge.net/ */
#include <GL/glut.h> /* GL Utilities Toolkit, http://freeglut.sourceforge.net/ */

void Exit(const char *where,const char *why) {
	fprintf (stderr, "FATAL OpenGL Error in %s: %s\n", where, why);
	abort();	
}

// Check GL for errors after the last command
void Check(const char *where) {
	GLenum e=glGetError();
	if (e==GL_NO_ERROR) return;
	const GLubyte *errString = gluErrorString(e);
	Exit(where, (const char *)errString);
}

// Draw a little quadrilateral onscreen
void drawBox(void) {
	glBegin(GL_TRIANGLE_FAN);
	glColor4f(1,1,1,1);  
	glTexCoord2f(0,0);   glVertex4f(-1,-1,0,1);  
	glTexCoord2f(0,1);   glVertex4f(-1,+1,0,1);  
	glTexCoord2f(1,1);   glVertex4f(+1,+1,0,1);  
	glTexCoord2f(1,0);   glVertex4f(+1,-1,0,1);
	glEnd();
}

/******** Program Initialization *********/
int win_w, win_h; /**< Width and height (in pixels) of our window at last resize */

/* GLUT calls this routine every time our window needs to be redrawn */
void main_display(void) {
	Check("Entering display routine");

// Clear the color and depth buffers
	glClearColor(0.2,0.3,0.4,1.0); /* set clear color */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); /* actually clear color and depth */
	
// Draw stuff onscreen
	GLuint texNum=0;
	glGenTextures(1,&texNum); /* texNum is now a unique "texture name"--probably the value 1! */
	glActiveTextureARB(GL_TEXTURE0_ARB); /* we're using texture unit zero (this is the default, actually) */
	glBindTexture(GL_TEXTURE_2D,texNum); /* texNum is now the current texture--all texture ops now refer to texNum */

	/* Fill the currently bound texture (texNum) with pixel data--red, green, blue, white. */
	unsigned char texData[]={0xff,0x00,0x00,   0x00,0xff,0x00,   0x00,0x00, 0xff,   0xff,0xff,0xff};
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGB8,4,1,0,GL_RGB,GL_UNSIGNED_BYTE,texData);

	/* Set the magnification, minification, and wrap modes on the current texture (texNum) */
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);

	// Draw geometry, using currently bound texture */
	glEnable(GL_TEXTURE_2D); /*<- fixed-function (not GLSL) texture enable */
	drawBox();

	/* Throw away all the texture data bound to texNum.  Note you can have many textures at once, so only throw out stuff you're really done using! */
	glDeleteTextures(1,&texNum);

	
	
// Display the stuff drawn so far to the screen:
	glutSwapBuffers();
	Check("Leaving display routine");
}

/* GLUT calls this routine every time our window changes size */
void main_reshape(int w, int h) {
	win_w=w; win_h=h;
	glViewport(0, 0, (GLsizei)w, (GLsizei)(h));
}

int main(int argc, char** argv)
{
	glutInit(&argc,argv);

	// Ask for a back buffer, color buffer, and Z buffer in a window.
	int mode=GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH;
	glutInitDisplayMode (mode);
	int main_window = glutCreateWindow("Bare-metal OpenGL Demo");
	
	// Turn on OpenGL extensions, like programmable shaders
	//   WARNING: you MUST call glutCreateWindow before doing this!
	//   If you don't call glewInit, or call it wrong, you'll segfault!
	glewInit();

	// Ask GLUT to call us when we need to redraw
	glutDisplayFunc (main_display);
	glutReshapeFunc (main_reshape);
	
	// Process events until user closes our window...
	glutMainLoop();
	
	return 0;
}
